==> dapi-2.14.2.tar.gz: ios/Classes/DPCCrossSDK.m <==
//
//  DPCCrossSDK.m
//  DapiSDK
//
//  Created by MSZ on 11/11/2022.
//

#import "DPCCrossSDK.h"
#define NULL_TO_NIL(obj) ({ __typeof__ (obj) __obj = (obj); [__obj isKindOfClass:[NSNull class]] ? nil : obj; })


@implementation UIViewController (VisibleViewController)
- (UIViewController *)visibleViewController {
    BOOL isNavigationController = [self isKindOfClass:[UINavigationController class]];
    BOOL isTabBarController = [self isKindOfClass:[UITabBarController class]];
    UIViewController *presentedViewController = self.presentedViewController;
    //    BOOL isAlertController = [self isKindOfClass:[UIAlertController class]];

    if (isNavigationController) {
        UINavigationController *navigationController = (UINavigationController *)self;
        return navigationController.topViewController.visibleViewController;
    } else if (isTabBarController) {
        UITabBarController *tabBarController = (UITabBarController *)self;
        return tabBarController.selectedViewController.visibleViewController;
    } else if (presentedViewController) {
        return self.presentedViewController.visibleViewController;
    }
    else {
        return self;
    }
}


@end

@implementation DPCCrossSDK

+ (UIViewController *)topViewController {
    return [UIApplication sharedApplication].delegate.window.rootViewController.visibleViewController;
}

+ (DAPIConfigurations *)setNativeConfigurations:(NSDictionary<NSString *, id> *)configs {
    NSArray *countries = [configs objectForKey:@"countries"];

    DAPIConfigurations *configurations = [[DAPIConfigurations alloc] init];
    configurations.showTransferErrorResult = YES;
    configurations.showLogos = YES;
    configurations.showTransferSuccessfulResult = YES;
    [DPC.sharedInstance setCountries: countries] ;
    [DPC.sharedInstance setConfigurations:configurations];
    NSDictionary<NSString *, id> *endPointExtraHeaderFields = NULL_TO_NIL([configs objectForKey:@"extraHeaderFields"]);
    NSDictionary<NSString *, id> *endPointExtraBody = NULL_TO_NIL([configs objectForKey:@"extraBody"]);

    if (endPointExtraHeaderFields != (id)[NSNull null] && endPointExtraHeaderFields != nil) {
        [DPC.sharedInstance setEndpointExtraHeaderFields:@{DPCEndpointAll: endPointExtraHeaderFields}];
    }
    if (endPointExtraBody != (id)[NSNull null] && endPointExtraBody != nil ) {
        [DPC.sharedInstance setEndpointExtraBody:@{DPCEndpointAll: endPointExtraBody}];
    }
    NSNumber *showLogos = NULL_TO_NIL([configs objectForKey:@"showLogos"]);
    NSNumber *showTransferSuccessfulResult = NULL_TO_NIL([configs objectForKey:@"showTransferSuccessfulResult"]);
    NSNumber *showTransferErrorResult = NULL_TO_NIL([configs objectForKey:@"showTransferErrorResult"]);

    if (showLogos != (id)[NSNull null] && showLogos != nil  ) {
        configurations.showLogos = showLogos.boolValue;
    }
    if (showTransferSuccessfulResult != (id)[NSNull null] && showTransferSuccessfulResult != nil  ) {
        configurations.showTransferSuccessfulResult = showTransferSuccessfulResult.boolValue;
    }

    if (showTransferErrorResult != (id)[NSNull null] && showTransferErrorResult != nil  ) {
        configurations.showTransferErrorResult = showTransferErrorResult.boolValue;
    }
    
    NSString *language = NULL_TO_NIL([configs objectForKey:@"language"]);
    DAPILanguage dapiLanguage = DAPILanguageEn;
    if (language != (id)[NSNull null] &&
        language != nil &&
        [language isEqualToString: @"ar"]) {
        dapiLanguage = DAPILanguageAr;
    }
    configurations.language = dapiLanguage;
    
    NSDictionary *themeDic = NULL_TO_NIL([configs objectForKey:@"theme"]);
    if (themeDic == (id)[NSNull null] && themeDic != nil  ) {
        return configurations;
    }
    NSString *theme = NULL_TO_NIL([themeDic objectForKey:@"enforceTheme"]);
    if (theme != (id)[NSNull null] && theme != nil && [theme isEqualToString: @"light"]){
        [Dapi.shared.uiCustomization setTheme: DAPIThemeLight];
    } else if (theme != (id)[NSNull null] && theme != nil && [theme isEqualToString: @"dark"]){
        [Dapi.shared.uiCustomization setTheme: DAPIThemeDark];
    } else {
        [Dapi.shared.uiCustomization setTheme: DAPIThemeDynamic];
    }
    NSDictionary *primaryColor = NULL_TO_NIL([themeDic objectForKey:@"primaryColor"]);
    if (primaryColor != (id)[NSNull null] && primaryColor != nil ) {
        NSString *darkModeHexa = [primaryColor objectForKey:@"darkMode"];
        if (darkModeHexa != (id)[NSNull null] && darkModeHexa != nil) {
            [Dapi.shared.uiCustomization.primaryColor setDarkModeHexaCode:darkModeHexa];
        }
        NSString *lightModeHexa = [primaryColor objectForKey:@"lightMode"];
        if (lightModeHexa != (id)[NSNull null] && lightModeHexa != nil) {
            [Dapi.shared.uiCustomization.primaryColor setLightModeHexaCode:lightModeHexa];
        }
    }

    return configurations;
}



+ (NSDictionary<NSString *, id> *)jsConfigurations {
    NSString *environment = @"sandbox";
    if (Dapi.shared.environment == DAPIEnvironmentProduction ) {
        environment = @"production";
    }
    return @{
        @"countries": DPC.sharedInstance.countries ?: [NSNull null],
        @"environment": environment ?: [NSNull null],
        @"endPointExtraQueryItems":  @{@"":@[@""]},
        @"endPointExtraHeaderFields": DPC.sharedInstance.endpointExtraHeaderFields ?: [NSNull null],
        @"endPointExtraBody": DPC.sharedInstance.endpointExtraBody ?: [NSNull null],
        @"showCloseButton": [NSNumber numberWithBool: NO],
        @"showLogos": [NSNumber numberWithBool:Dapi.shared.configurations.showLogos],
        @"showAddButton": [NSNumber numberWithBool:NO],
        @"showTransferSuccessfulResult": [NSNumber numberWithBool:Dapi.shared.configurations.showTransferSuccessfulResult],
        @"showTransferErrorResult": [NSNumber numberWithBool:Dapi.shared.configurations.showTransferErrorResult]
    };

}
+ (DAPIBankConnection *)bankConnectionForUserID:(NSString *)userID {
    __block DAPIBankConnection *bankConnection;
    [DPC.sharedInstance.getConnections enumerateObjectsUsingBlock:^(DAPIBankConnection * _Nonnull obj,
                                                                    NSUInteger idx,
                                                                    BOOL * _Nonnull stop) {
        if ([obj.userID isEqualToString:userID]) {
            *stop = YES;
            bankConnection = obj;
        }
    }];
    return  bankConnection;
}

+ (NSString *)jsonRepresentationOfError:(NSError *)error
                                accountID:(NSString *)accountID
                            operationID:(NSString *)operationID {

    NSDictionary *jsonError = @{
        @"error": error.localizedDescription ?: [NSNull null],
        @"account": [[NSString alloc] initWithFormat:@"AccountID %@", accountID],
        @"operationID": operationID ?: [NSNull null]
    };

    NSError *convertingError;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:jsonError
                                                       options:NSJSONWritingPrettyPrinted
                                                         error:&convertingError];
    if (convertingError) {
        return [NSString stringWithFormat:@"{\"error\": %@, \"account\": null}", error.dapiErrorMessage];
    } else {
        NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
        return jsonString;
    }
}

+ (DAPIBeneficiary *)nativeBeneficiaryInfoFromDictionary:(NSDictionary<NSString *,id> *)beneficiaryInfoDictionary {
    DAPIBeneficiary *beneficiary = [[DAPIBeneficiary alloc] init];
    NSDictionary<NSString *, id> *lineAddressDictionary = [beneficiaryInfoDictionary objectForKey:@"linesAddress"];
    DAPILineAddress *lineAddress = [[DAPILineAddress alloc] init];
    lineAddress.line1 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line1"]);
    lineAddress.line2 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line2"]);
    lineAddress.line3 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line3"]);
    beneficiary.linesAddress =  lineAddress;
    beneficiary.accountNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"accountNumber"]);
    beneficiary.name = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"name"]);
    beneficiary.bankName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"bankName"]);
    beneficiary.swiftCode = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"swiftCode"]);
    beneficiary.iban = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"iban"]);
    beneficiary.phoneNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"phoneNumber"]);
    beneficiary.country = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"country"]);
    beneficiary.branchAddress = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"branchAddress"]);
    beneficiary.branchName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"branchName"]);
    return beneficiary;
}

+ (DAPIWireBeneficiary *)nativeWireBeneficiaryInfoFromDictionary:(NSDictionary<NSString *,id> *)beneficiaryInfoDictionary {
    DAPIWireBeneficiary *beneficiary = [[DAPIWireBeneficiary alloc] init];
    NSDictionary<NSString *, id> *lineAddressDictionary = [beneficiaryInfoDictionary objectForKey:@"linesAddress"];
    DAPILineAddress *lineAddress = [[DAPILineAddress alloc] init];
    lineAddress.line1 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line1"]);
    lineAddress.line2 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line2"]);
    lineAddress.line3 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line3"]);
    beneficiary.linesAddress = lineAddress;
    beneficiary.accountNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"accountNumber"]);
    beneficiary.name = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"name"]);
    beneficiary.country = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"country"]);
    beneficiary.receiverType = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"receiverType"]);
    beneficiary.routingNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"routingNumber"]);
    beneficiary.nickname = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"nickname"]);
    beneficiary.receiverAccountType = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"receiverAccountType"]);
    beneficiary.firstName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"firstName"]);
    beneficiary.lastName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"lastName"]);
    beneficiary.zipCode = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"zipCode"]);
    beneficiary.state = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"state"]);
    beneficiary.city = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"city"]);
    return beneficiary;
}

@end

==> dapi-2.22.0.tar.gz: ios/Classes/DPCCrossSDK.m <==
//
//  DPCCrossSDK.m
//  DapiSDK
//
//  Created by MSZ on 11/11/2022.
//

#import "DPCCrossSDK.h"
#define NULL_TO_NIL(obj) ({ __typeof__ (obj) __obj = (obj); [__obj isKindOfClass:[NSNull class]] ? nil : obj; })


@implementation UIViewController (VisibleViewController)
- (UIViewController *)visibleViewController {
    BOOL isNavigationController = [self isKindOfClass:[UINavigationController class]];
    BOOL isTabBarController = [self isKindOfClass:[UITabBarController class]];
    UIViewController *presentedViewController = self.presentedViewController;
    //    BOOL isAlertController = [self isKindOfClass:[UIAlertController class]];

    if (isNavigationController) {
        UINavigationController *navigationController = (UINavigationController *)self;
        return navigationController.topViewController.visibleViewController;
    } else if (isTabBarController) {
        UITabBarController *tabBarController = (UITabBarController *)self;
        return tabBarController.selectedViewController.visibleViewController;
    } else if (presentedViewController) {
        return self.presentedViewController.visibleViewController;
    }
    else {
        return self;
    }
}


@end

@implementation DPCCrossSDK

+ (DAPIConfigurations *)setNativeConfigurations:(NSDictionary<NSString *, id> *)configs {
    NSArray *countries = [configs objectForKey:@"countries"];

    DAPIConfigurations *configurations = [[DAPIConfigurations alloc] init];
    configurations.showTransferErrorResult = YES;
    configurations.showLogos = YES;
    configurations.showTransferSuccessfulResult = YES;
    [DPC.sharedInstance setCountries: countries] ;
    [DPC.sharedInstance setConfigurations:configurations];
    NSDictionary<NSString *, id> *endPointExtraHeaderFields = NULL_TO_NIL([configs objectForKey:@"extraHeaderFields"]);
    NSDictionary<NSString *, id> *endPointExtraBody = NULL_TO_NIL([configs objectForKey:@"extraBody"]);

    if (endPointExtraHeaderFields != (id)[NSNull null] && endPointExtraHeaderFields != nil) {
        [DPC.sharedInstance setEndpointExtraHeaderFields:@{DPCEndpointAll: endPointExtraHeaderFields}];
    }
    if (endPointExtraBody != (id)[NSNull null] && endPointExtraBody != nil ) {
        [DPC.sharedInstance setEndpointExtraBody:@{DPCEndpointAll: endPointExtraBody}];
    }
    NSNumber *showLogos = NULL_TO_NIL([configs objectForKey:@"showLogos"]);
    NSNumber *showTransferSuccessfulResult = NULL_TO_NIL([configs objectForKey:@"showTransferSuccessfulResult"]);
    NSNumber *showTransferErrorResult = NULL_TO_NIL([configs objectForKey:@"showTransferErrorResult"]);

    if (showLogos != (id)[NSNull null] && showLogos != nil  ) {
        configurations.showLogos = showLogos.boolValue;
    }
    if (showTransferSuccessfulResult != (id)[NSNull null] && showTransferSuccessfulResult != nil  ) {
        configurations.showTransferSuccessfulResult = showTransferSuccessfulResult.boolValue;
    }

    if (showTransferErrorResult != (id)[NSNull null] && showTransferErrorResult != nil  ) {
        configurations.showTransferErrorResult = showTransferErrorResult.boolValue;
    }
    
    NSString *language = NULL_TO_NIL([configs objectForKey:@"language"]);
    DAPILanguage dapiLanguage = DAPILanguageEn;
    if (language != (id)[NSNull null] &&
        language != nil &&
        [language isEqualToString: @"ar"]) {
        dapiLanguage = DAPILanguageAr;
    }
    configurations.language = dapiLanguage;
    
    NSDictionary *themeDic = NULL_TO_NIL([configs objectForKey:@"theme"]);
    if (themeDic == (id)[NSNull null] && themeDic != nil  ) {
        return configurations;
    }
    NSString *theme = NULL_TO_NIL([themeDic objectForKey:@"enforceTheme"]);
    if (theme != (id)[NSNull null] && theme != nil && [theme isEqualToString: @"light"]){
        [Dapi.shared.uiCustomization setTheme: DAPIThemeLight];
    } else if (theme != (id)[NSNull null] && theme != nil && [theme isEqualToString: @"dark"]){
        [Dapi.shared.uiCustomization setTheme: DAPIThemeDark];
    } else {
        [Dapi.shared.uiCustomization setTheme: DAPIThemeDynamic];
    }
    NSDictionary *primaryColor = NULL_TO_NIL([themeDic objectForKey:@"primaryColor"]);
    if (primaryColor != (id)[NSNull null] && primaryColor != nil ) {
        NSString *darkModeHexa = [primaryColor objectForKey:@"darkMode"];
        if (darkModeHexa != (id)[NSNull null] && darkModeHexa != nil) {
            [Dapi.shared.uiCustomization.primaryColor setDarkModeHexaCode:darkModeHexa];
        }
        NSString *lightModeHexa = [primaryColor objectForKey:@"lightMode"];
        if (lightModeHexa != (id)[NSNull null] && lightModeHexa != nil) {
            [Dapi.shared.uiCustomization.primaryColor setLightModeHexaCode:lightModeHexa];
        }
    }

    return configurations;
}



+ (NSDictionary<NSString *, id> *)jsConfigurations {
    NSString *environment = @"sandbox";
    if (Dapi.shared.environment == DAPIEnvironmentProduction ) {
        environment = @"production";
    }
    return @{
        @"countries": DPC.sharedInstance.countries ?: [NSNull null],
        @"environment": environment ?: [NSNull null],
        @"endPointExtraQueryItems":  @{@"":@[@""]},
        @"endPointExtraHeaderFields": DPC.sharedInstance.endpointExtraHeaderFields ?: [NSNull null],
        @"endPointExtraBody": DPC.sharedInstance.endpointExtraBody ?: [NSNull null],
        @"showCloseButton": [NSNumber numberWithBool: NO],
        @"showLogos": [NSNumber numberWithBool:Dapi.shared.configurations.showLogos],
        @"showAddButton": [NSNumber numberWithBool:NO],
        @"showTransferSuccessfulResult": [NSNumber numberWithBool:Dapi.shared.configurations.showTransferSuccessfulResult],
        @"showTransferErrorResult": [NSNumber numberWithBool:Dapi.shared.configurations.showTransferErrorResult]
    };

}
+ (DAPIBankConnection *)bankConnectionForUserID:(NSString *)userID {
    __block DAPIBankConnection *bankConnection;
    [DPC.sharedInstance.getConnections enumerateObjectsUsingBlock:^(DAPIBankConnection * _Nonnull obj,
                                                                    NSUInteger idx,
                                                                    BOOL * _Nonnull stop) {
        if ([obj.userID isEqualToString:userID]) {
            *stop = YES;
            bankConnection = obj;
        }
    }];
    return  bankConnection;
}

+ (NSString *)jsonRepresentationOfError:(NSError *)error
                                accountID:(NSString *)accountID {

    NSDictionary *jsonError = @{
        @"message": [error dapiErrorMessage] ?: [NSNull null],
        @"type": [error dapiErrorType] ?: [NSNull null],
        @"statusCode": error.dapiStatusCode  ?: [NSNull null],
        @"operationID": error.dapiOperationID ?: [NSNull null],
        @"account": [[NSString alloc] initWithFormat:@"%@", accountID],
    };
    
    NSError *convertingError;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:jsonError
                                                       options:NSJSONWritingPrettyPrinted
                                                         error:&convertingError];
    if (convertingError) {
        return [NSString stringWithFormat:@"{\"message\": \"%@\", \"account\": \"%@\" }", error.dapiErrorMessage,
                [[NSString alloc] initWithFormat:@"%@", accountID]
        ];
    } else {
        NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
        return jsonString;
    }
}

+ (DAPIBeneficiary *)nativeBeneficiaryInfoFromDictionary:(NSDictionary<NSString *,id> *)beneficiaryInfoDictionary {
    DAPIBeneficiary *beneficiary = [[DAPIBeneficiary alloc] init];
    NSDictionary<NSString *, id> *lineAddressDictionary = [beneficiaryInfoDictionary objectForKey:@"linesAddress"];
    DAPILineAddress *lineAddress = [[DAPILineAddress alloc] init];
    lineAddress.line1 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line1"]);
    lineAddress.line2 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line2"]);
    lineAddress.line3 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line3"]);
    beneficiary.linesAddress =  lineAddress;
    beneficiary.accountNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"accountNumber"]);
    beneficiary.name = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"name"]);
    beneficiary.bankName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"bankName"]);
    beneficiary.swiftCode = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"swiftCode"]);
    beneficiary.iban = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"iban"]);
    beneficiary.phoneNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"phoneNumber"]);
    beneficiary.country = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"country"]);
    beneficiary.branchAddress = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"branchAddress"]);
    beneficiary.branchName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"branchName"]);
    beneficiary.nickname = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"nickname"]);
    return beneficiary;
}

+ (DAPIWireBeneficiary *)nativeWireBeneficiaryInfoFromDictionary:(NSDictionary<NSString *,id> *)beneficiaryInfoDictionary {
    DAPIWireBeneficiary *beneficiary = [[DAPIWireBeneficiary alloc] init];
    NSDictionary<NSString *, id> *lineAddressDictionary = [beneficiaryInfoDictionary objectForKey:@"linesAddress"];
    DAPILineAddress *lineAddress = [[DAPILineAddress alloc] init];
    lineAddress.line1 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line1"]);
    lineAddress.line2 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line2"]);
    lineAddress.line3 =  NULL_TO_NIL([lineAddressDictionary objectForKey:@"line3"]);
    beneficiary.linesAddress = lineAddress;
    beneficiary.accountNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"accountNumber"]);
    beneficiary.name = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"name"]);
    beneficiary.country = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"country"]);
    beneficiary.receiverType = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"receiverType"]);
    beneficiary.routingNumber = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"routingNumber"]);
    beneficiary.nickname = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"nickname"]);
    beneficiary.receiverAccountType = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"receiverAccountType"]);
    beneficiary.firstName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"firstName"]);
    beneficiary.lastName = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"lastName"]);
    beneficiary.zipCode = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"zipCode"]);
    beneficiary.state = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"state"]);
    beneficiary.city = NULL_TO_NIL([beneficiaryInfoDictionary objectForKey:@"city"]);
    return beneficiary;
}


+(UIWindow *) returnWindowWithWindowLevelNormal
{
    NSArray *windows = [UIApplication sharedApplication].windows;
    for(UIWindow *topWindow in windows)
    {
        if (topWindow.windowLevel == UIWindowLevelNormal)
            return topWindow;
    }
    return [UIApplication sharedApplication].keyWindow;
}

+ (UIViewController *)topViewController {
    UIWindow *topWindow = [UIApplication sharedApplication].keyWindow;
    if (topWindow.windowLevel != UIWindowLevelNormal)
    {
        topWindow = [self returnWindowWithWindowLevelNormal];
    }

    UIViewController *topController = topWindow.rootViewController;
    if(topController == nil)
    {
        topWindow = [UIApplication sharedApplication].delegate.window;
        if (topWindow.windowLevel != UIWindowLevelNormal)
        {
            topWindow = [self returnWindowWithWindowLevelNormal];
        }
        topController = topWindow.rootViewController;
    }

    while(topController.presentedViewController)
    {
        topController = topController.presentedViewController;
    }

    if([topController isKindOfClass:[UINavigationController class]])
    {
        UINavigationController *nav = (UINavigationController*)topController;
        topController = [nav.viewControllers lastObject];

        while(topController.presentedViewController)
        {
            topController = topController.presentedViewController;
        }
    }

    return topController;
}
@end
